@@ -118,7 +118,7 @@ public:
   NS_IMETHOD GetValue(nsAString& aValue);
 
   // nsIXTFElement overrides
-  NS_IMETHOD OnCreated(nsIXTFBindableElementWrapper *aWrapper);
+  NS_IMETHOD OnCreated(nsIXTFElementWrapper *aWrapper);
   NS_IMETHOD WillChangeDocument(nsIDOMDocument *aNewDocument);
   NS_IMETHOD OnDestroyed();
   NS_IMETHOD ParentChanged(nsIDOMElement *aNewParent);
@@ -135,7 +135,7 @@ public:
   NS_DECL_NSIXFORMSACTIONMODULEELEMENT
 
   virtual nsIDOMElement* GetElement() { return mElement; }
-  virtual nsresult HandleSingleAction(nsIDOMEvent *aEvent,
+  virtual nsresult HandleSingleAction(nsIDOMEvent* aEvent,
                                       nsIXFormsActionElement *aParentAction);
   // Start the timer, which is used to set the message visible
   void StartEphemeral();
@@ -198,6 +198,13 @@ private:
    */
   PRBool IsEphemeral();
 
+  /** Set context info for events.
+   *
+   * @param aName     Name of the context property.
+   * @param aValue    Value of the context property.
+   */
+  nsresult SetContextInfo(const char *aName, const nsAString &aValue);
+
   MessageType          mType;
 
   // The position of the ephemeral message
@@ -210,6 +217,9 @@ private:
   StopType             mStopType;
   nsCString            mSrcAttrText;
   PRBool               mDoneAddingChildren;
+  // Context Info for events.
+  nsCOMArray<nsIXFormsContextInfo> mContextInfo;
+  nsString             mSrc;
 };
 
 NS_IMPL_ADDREF_INHERITED(nsXFormsMessageElement, nsXFormsDelegateStub)
@@ -219,6 +229,7 @@ NS_INTERFACE_MAP_BEGIN(nsXFormsMessageElement)
   NS_INTERFACE_MAP_ENTRY(nsIChannelEventSink)
   NS_INTERFACE_MAP_ENTRY(nsIInterfaceRequestor)
   NS_INTERFACE_MAP_ENTRY(nsIStreamListener)
+  NS_INTERFACE_MAP_ENTRY(nsIRequestObserver)
   NS_INTERFACE_MAP_ENTRY(nsIXFormsActionModuleElement)
   NS_INTERFACE_MAP_ENTRY(nsIDOMEventListener)
 NS_INTERFACE_MAP_END_INHERITING(nsXFormsDelegateStub)
@@ -226,7 +237,7 @@ NS_INTERFACE_MAP_END_INHERITING(nsXFormsDelegateStub)
 // nsIXTFElement
 
 NS_IMETHODIMP
-nsXFormsMessageElement::OnCreated(nsIXTFBindableElementWrapper *aWrapper)
+nsXFormsMessageElement::OnCreated(nsIXTFElementWrapper *aWrapper)
 {
   nsresult rv = nsXFormsDelegateStub::OnCreated(aWrapper);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -249,8 +260,8 @@ nsXFormsMessageElement::WillChangeDocument(nsIDOMDocument *aNewDocument)
     nsCOMPtr<nsIDocument> doc(do_QueryInterface(mDocument));
     if (doc) {
       nsXFormsMessageElement *msg =
-        NS_STATIC_CAST(nsXFormsMessageElement*,
-                       doc->GetProperty(nsXFormsAtoms::messageProperty));
+        static_cast<nsXFormsMessageElement*>
+                   (doc->GetProperty(nsXFormsAtoms::messageProperty));
       if (msg == this)
         doc->UnsetProperty(nsXFormsAtoms::messageProperty);
     }
@@ -445,7 +456,7 @@ nsXFormsMessageElement::DoneAddingChildren()
 }
 
 NS_IMETHODIMP
-nsXFormsMessageElement::HandleAction(nsIDOMEvent *aEvent,
+nsXFormsMessageElement::HandleAction(nsIDOMEvent* aEvent,
                                      nsIXFormsActionElement *aParentAction)
 {
   return nsXFormsActionModuleBase::DoHandleAction(this, aEvent, aParentAction);
@@ -467,7 +478,13 @@ nsXFormsMessageElement::HandleSingleAction(nsIDOMEvent *aEvent,
     nsCOMPtr<nsIModelElementPrivate> modelPriv =
       nsXFormsUtils::GetModel(mElement);
     nsCOMPtr<nsIDOMNode> model = do_QueryInterface(modelPriv);
-    nsXFormsUtils::DispatchEvent(model, eEvent_LinkError);
+
+    // Context Info: 'resource-uri'
+    // The URI associated with the failed link.
+    SetContextInfo("resource-uri", mSrc);
+
+    nsXFormsUtils::DispatchEvent(model, eEvent_LinkError, nsnull, nsnull,
+                                 &mContextInfo);
     return NS_OK;
   }
 
@@ -586,8 +603,8 @@ nsXFormsMessageElement::HandleEphemeralMessage(nsIDOMDocument* aDoc,
       return NS_OK;
 
     nsXFormsMessageElement *msg =
-    NS_STATIC_CAST(nsXFormsMessageElement*,
-                   doc->GetProperty(nsXFormsAtoms::messageProperty));
+    static_cast<nsXFormsMessageElement*>
+               (doc->GetProperty(nsXFormsAtoms::messageProperty));
     if (msg == this) {
       if (eventType.EqualsLiteral("xforms-moz-hint-off")) {
         if (mEphemeralTimer) {
@@ -684,8 +701,7 @@ nsXFormsMessageElement::HandleModalAndModelessMessage(nsIDOMDocument* aDoc,
     nsCOMPtr<nsIDocument> doc(do_QueryInterface(aDoc));
     NS_ENSURE_STATE(doc);
     nsCOMPtr<nsIURI> uri;
-    NS_NewURI(getter_AddRefs(uri), src, doc->GetDocumentCharacterSet().get(),
-              doc->GetDocumentURI());
+    nsXFormsUtils::GetNewURI(doc, src, getter_AddRefs(uri));
     NS_ENSURE_STATE(uri);
     nsCAutoString uriSpec;
     uri->GetSpec(uriSpec);
@@ -711,7 +727,7 @@ nsXFormsMessageElement::HandleModalAndModelessMessage(nsIDOMDocument* aDoc,
                                getter_AddRefs(htmlEl));
     NS_ENSURE_SUCCESS(rv, rv);
     htmlEl->SetAttribute(NS_LITERAL_STRING("style"),
-                       NS_LITERAL_STRING("background-color: -moz-Dialog;"));
+                         NS_LITERAL_STRING("background-color: -moz-Dialog;"));
     ddoc->AppendChild(htmlEl, getter_AddRefs(tmp));
 
     nsCOMPtr<nsIDOMElement> bodyEl;
@@ -805,7 +821,7 @@ void
 sEphemeralCallbackShow(nsITimer *aTimer, void *aListener)
 {
   nsXFormsMessageElement* self =
-    NS_STATIC_CAST(nsXFormsMessageElement*, aListener);
+    static_cast<nsXFormsMessageElement*>(aListener);
   if (self)
     self->ShowEphemeral();
 }
@@ -814,7 +830,7 @@ void
 sEphemeralCallbackHide(nsITimer *aTimer, void *aListener)
 {
   nsXFormsMessageElement* self =
-    NS_STATIC_CAST(nsXFormsMessageElement*, aListener);
+    static_cast<nsXFormsMessageElement*>(aListener);
   if (self)
     self->HideEphemeral();
 }
@@ -823,7 +839,7 @@ void
 sEphemeralCallbackResetPosition(nsITimer *aTimer, void *aListener)
 {
   nsXFormsMessageElement* self =
-    NS_STATIC_CAST(nsXFormsMessageElement*, aListener);
+    static_cast<nsXFormsMessageElement*>(aListener);
   if (self)
     self->ResetEphemeralPosition();
 }
@@ -885,8 +901,8 @@ nsXFormsMessageElement::HideEphemeral()
   if (!doc)
     return;
   nsXFormsMessageElement *msg =
-    NS_STATIC_CAST(nsXFormsMessageElement*,
-                   doc->GetProperty(nsXFormsAtoms::messageProperty));
+    static_cast<nsXFormsMessageElement*>
+               (doc->GetProperty(nsXFormsAtoms::messageProperty));
   if (msg && msg != this) {
     msg->HideEphemeral();
     return;
@@ -929,14 +945,16 @@ nsXFormsMessageElement::TestExternalFile()
   if (src.IsEmpty()) {
     return NS_OK;
   }
+  // Remember the src attribute so we can set it in the context info
+  // for the xforms-link-error event if the link fails.
+  mSrc = src;
 
   nsCOMPtr<nsIDOMDocument> domDoc;
   mElement->GetOwnerDocument(getter_AddRefs(domDoc));
   nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
   NS_ENSURE_STATE(doc);
   nsCOMPtr<nsIURI> uri;
-  NS_NewURI(getter_AddRefs(uri), src, doc->GetDocumentCharacterSet().get(),
-            doc->GetDocumentURI());
+  nsXFormsUtils::GetNewURI(doc, src, getter_AddRefs(uri));
   NS_ENSURE_STATE(uri);
 
   if (!nsXFormsUtils::CheckConnectionAllowed(mElement, uri)) {
@@ -993,6 +1011,9 @@ nsXFormsMessageElement::TestExternalFile()
     nsXFormsUtils::ReportError(NS_LITERAL_STRING("externalLink1Error"),
                                strings, 2, mElement, mElement);
     mStopType = eStopType_LinkError;
+    // Remember the src attribute so we can set it in the context info
+    // for the xforms-link-error event.
+    mSrc = src;
     return NS_ERROR_FAILURE;
   }
 
@@ -1103,6 +1124,9 @@ nsXFormsMessageElement::OnStartRequest(nsIRequest *aRequest,
       nsXFormsUtils::ReportError(NS_LITERAL_STRING("externalLink2Error"),
                                  strings, 2, mElement, mElement);
       mStopType = eStopType_LinkError;
+      // Remember the src attribute so we can set it in the context info
+      // for the xforms-link-error event.
+      mSrc = src;
     }
 
     AddRemoveExternalResource(PR_FALSE);
@@ -1129,6 +1153,9 @@ nsXFormsMessageElement::OnStartRequest(nsIRequest *aRequest,
       nsXFormsUtils::ReportError(NS_LITERAL_STRING("externalLink2Error"),
                                  strings, 2, mElement, mElement);
       mStopType = eStopType_LinkError;
+      // Remember the src attribute so we can set it in the context info
+      // for the xforms-link-error event.
+      mSrc = src;
     }
   }
 
@@ -1221,7 +1248,7 @@ nsXFormsMessageElement::AddRemoveExternalResource(PRBool aAdd)
     }
   }
   doc->SetProperty(nsXFormsAtoms::externalMessagesProperty,
-                   NS_REINTERPRET_CAST(void *, loadingMessages), nsnull);
+                   reinterpret_cast<void *>(loadingMessages), nsnull);
 
   if (!loadingMessages) {
     // no outstanding loads left, let the model in the document know in case
@@ -1256,6 +1283,24 @@ PRBool nsXFormsMessageElement::IsEphemeral()
   return level.Equals(NS_LITERAL_STRING("ephemeral"));
 }
 
+nsresult
+nsXFormsMessageElement::SetContextInfo(const char *aName, const nsAString &aValue)
+{
+  nsCOMPtr<nsXFormsContextInfo> contextInfo = new nsXFormsContextInfo(mElement);
+  NS_ENSURE_TRUE(contextInfo, NS_ERROR_OUT_OF_MEMORY);
+  contextInfo->SetStringValue(aName, aValue);
+  mContextInfo.AppendObject(contextInfo);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsXFormsMessageElement::GetCurrentEvent(nsIDOMEvent **aEvent)
+{
+  NS_IF_ADDREF(*aEvent = mCurrentEvent);
+  return NS_OK;
+}
+
 NS_HIDDEN_(nsresult)
 NS_NewXFormsMessageElement(nsIXTFElement **aResult)
 {
